
#include "Constants.h"
#include "IOpins.h"


int volts;                                         // battery voltage
int pause=-1;                                      // a value of -1 means the battery voltage is ok, 0 and 1 are used to flash LED on D13 when the battery is flat
byte pen;
byte turn;

int lspeed;                                        // left  speed -255 to +255 (negative values are reverse)
int rspeed;                                        // right speed -255 to +255 (negative values are reverse)
int dspeed=80;                                     // draw  speed  +50 to +200

// wheel encoders counters
volatile int rcount;
volatile int lcount;


String message = "";


void setup()
{
  attachInterrupt(0,Rcount,CHANGE);
  attachInterrupt(1,Lcount,CHANGE);
  
  Serial.begin(9600);                             // initialize serial interface - 9600 baud
  
  pinMode(Ldirpin,OUTPUT);                         // set left  direction pin as an output pin
  pinMode(Rdirpin,OUTPUT);                         // set right direction pin as an output pin
  pinMode(Srvopin,OUTPUT);                         // set servo  control  pin as an output pin
  
  digitalWrite(2,1);                               // enable pullup resistor on D2
  digitalWrite(3,1);                               // enable pullup resistor on D3
  
  Up();                                            // start the program with the pen raised
  
  //--------------------------------------------------------------------------- //

  pinMode(7,OUTPUT);                               // left  motor direction           
  pinMode(8,OUTPUT);                               // right motor direction
  pinMode(9,OUTPUT);                               // left  motor PWM
  pinMode(10,OUTPUT);                              // right motor PWM
  pinMode(13,OUTPUT);
    
  digitalWrite(13,1);                              // turn on D13 LED as indicator that program is running
  

}

void loop()
{ 
  volts=analogRead(Battery)/10;
  
  if(volts<lowbat || pause!=-1)                    // once voltage falls below lowbat value
  {
    lspeed=0;                                      // stop left  motor
    rspeed=0;                                      // stop right motor
    Motors();                                      // update motor control pins
    if (pause<0) Up();                             // make sure pen is raised when battery first falls below lowbat value
    pause++;                                       // increment pause
    if(pause>1) pause=0;                           // toggle pause between 0 and 1 once battery is flat
    digitalWrite(13,pause);                        // flash LED on D13
    delay(200);                                    // set flashing LED speed
    return;                                        // reset loop
  }
                                                   // current is proportional to voltage and power = voltage x current
                                                   // As the battery voltage drops dspeed increases to maintain a constant speed
  
  dspeed=26214/volts*10/volts;                     // draw speed power correction factor=100% when battery=5V (analog reading = 512)
  dspeed=dspeed*8/10;                              // adjust speed to reduce overshoot
 
   
 // -------------------------------- handle serial comms --------------------------------
 
 
  int i=0;
  char commandbuffer[100];

  if(Serial.available()){
     delay(100);
     while( Serial.available() && i< 99) {
        commandbuffer[i++] = Serial.read();
     }
     commandbuffer[i++]='\0';
  }

  if(i>0)
  {
    message = (char*)commandbuffer;
    message.toUpperCase();
    message.trim();
  }
 
 
 
  // (pen) DOWN
  if(message == "D") {
    boolean executed = false;
    while(!executed){
      executed = Down();
    }
    //if (executed) 
    Serial.print('A');
    message = "";
  }
  // (pen) UP
  if(message == "U") {
    boolean executed = false;
    while(!executed){
      executed = Up();
    }
    //if (executed) 
    Serial.print('A');
    message = "";
  }

  // move back and forward 
  if( message.startsWith("G") ){
    boolean executed = false;        
    String numbStr = "";
    for(int i=1; i <= message.length(); i++){ 
      numbStr += message[i];       
    }
    int len = numbStr.length();
    char numbArray[len];
    numbStr.toCharArray(numbArray, len);
    int numb = atoi(numbArray);
    while(!executed){
      executed = Go(numb);
    }
    //if (executed) 
    Serial.print('A');
    
    message = "";        
  }

  // turn around  
  if( message.startsWith("T") ){
    boolean executed = false;        
    String numbStr = "";
    for(int i=1; i <= message.length(); i++){ 
      numbStr += message[i];       
    }
    int len = numbStr.length();
    char numbArray[len];
    numbStr.toCharArray(numbArray, len);
    int numb = atoi(numbArray);
    while(!executed){
      executed = Turn(numb);
    }
    //if (executed) 
    Serial.print('A');

    message = "";
  }  
 

 // ---------------------------------------------
 
 
}


  





